local _G = getfenv(0)
local ADDON_NAME, addon = ...

local CompactRunes = LibStub("AceAddon-3.0"):GetAddon(addon.addonName)
local L = _G.LibStub("AceLocale-3.0"):GetLocale(addon.addonName, true)

local UnitBuff = addon.UnitBuff
local UnitDebuff = addon.UnitDebuff
local GetSpellCooldown = _G.GetSpellCooldown
local SpellIds = addon.SpellIds
local SpellNames = addon.SpellNames
local frames = addon.frames

function addon.SetupSpellAuras()
	addon.StandardDiseases = {
		["Blood"] = {
			[SpellNames["Blood Plague"]] = frames.bpBar,
		},
		["Frost"] = {
			[SpellNames["Frost Fever"]] = frames.ffBar,
		},
		["Unholy"] = {
			[SpellNames["Virulent Plague"]] = frames.vpBar,
		},
	}
	addon.petBuffs = {
		SpellNames["Dark Transformation"],
	}
end

function addon.GetRuneOrderMap()
	return {
		["Right"] = L["Right"],
		["Down"] = L["Down"],
		["Left"] = L["Left"],
		["Up"] = L["Up"],
	}
end

function addon.NeedsGhoul()
	local start, duration, enabled = GetSpellCooldown(SpellIds["Raise Dead"])
	return not GetTotemInfo(1) and duration <= 0
end

function CompactRunes:CheckSpells()
	self:UpdateSoulReaperEnabled()
end
  
-- If procs settings changed, update any necessary settings.
function CompactRunes:OnProcsChanged()
	self:UpdateSoulReaperEnabled()
end
  
-- Soul Reaper

function CheckSoulReaper()
	CompactRunes:CheckSoulReaper()
end
  
function CompactRunes:UpdateSoulReaperEnabled()
	local enabled = addon.db.profile.enabled[addon.currentSpec]
  
	addon.soulReaperEnabled = addon.currentSpec == "Unholy" and
	  addon.IsSpellKnown("Soul Reaper") and
	  enabled and enabled["Soul Reaper"]
	addon.soulReaperPercent = addon.db.profile.soulReaperPercentEnable and
	  addon.db.profile.soulReaperPercent or addon.SOUL_REAPER_PERCENT
  
	if addon.soulReaperEnabled then
	  addon:RegisterCallback("TargetHealth", "SoulReaper", CheckSoulReaper)
	else
	  addon:UnregisterCallback("TargetHealth", "SoulReaper")
	end
end
 
function CompactRunes:CheckSoulReaper(update)
	-- If below percent, known, and not on CD, show proc.
	if not addon.soulReaperEnabled then return end
  
	local changed = false
	local spellId = SpellIds["Soul Reaper"]
	local start, duration, enabled = GetSpellCooldown(spellId)
	local ready = (duration < 2)
	local usable = _G.IsUsableSpell(spellId)
	local show = (ready and usable and
	  addon.targetPercent <= addon.soulReaperPercent) and 1 or nil
  
	if show ~= self.currentProcs["Soul Reaper"] then
	  self.currentProcs["Soul Reaper"] = show
	  changed = true
	end
  
	if changed and update ~= false then
	  self:UpdateProcIcon()
	end
  
	return changed
  end

function CompactRunes:OnUnitAura(event, unit)
	local name, icon, count, dispelType, duration, expires,
		caster, isStealable, shouldConsolidate, spellId, canApplyAura,
		isBossDebuff, castByPlayer, value1, value2, value3

	if unit == "target" then
		if addon.currentSpec == "Unholy" then
			-- TODO: Function call taking spell name, bar, possibly the onUpdate.
			name, icon, count, dispelType, duration, expires,
			caster, isStealable, shouldConsolidate, spellId, canApplyAura,
			isBossDebuff, castByPlayer, value1, value2, value3
			= UnitDebuff("target", SpellNames["Festering Wound"], "PLAYER")
			local bar = frames.fwBar
			local fwProc = (count == nil or count < 5) and 1 or nil
			local changed = fwProc ~= self.currentProcs["Festering Wound < 5"]
			if changed then
				self.currentProcs["Festering Wound < 5"] = fwProc
				self:UpdateProcIcon()
			end
			-- if name and bar.db.enabled then
			-- 	bar.bar.active = true
			-- 	local timeLeft = expires - GetTime()
			-- 	bar.bar:SetMinMaxValues(0, duration)
			-- 	bar.bar.timer = timeLeft
			-- 	bar.bar:Show()
			-- 	bar.bar:SetScript("OnUpdate", addon.scripts.onUpdateTimerNoCounter)
			-- 	bar.bar.value:SetText(tostring(count))
			-- else
			-- 	bar.bar.active = false
			-- 	bar.bar.timer = 0
			-- 	bar.bar:SetScript("OnUpdate", nil)
			-- 	bar.bar:Hide()
			-- end
		end
	elseif unit == "player" then
		if self.db.profile.procs then
			if addon.currentSpec == "Blood" then
				local low = self.db.profile.boneShieldLow or addon.BONE_SHIELD_LOW
				name, icon, count, dispelType, duration, expires,
				caster, isStealable, shouldConsolidate, spellId, canApplyAura,
				isBossDebuff, castByPlayer, value1, value2, value3
				= UnitBuff("player", SpellNames["Bone Shield"], "PLAYER")
				local bsProc = (count == nil or count <= low) and (count or 1) or nil
				local changed = bsProc ~= self.currentProcs["Bone Shield <= X"]
				if changed then
					self.currentProcs["Bone Shield <= X"] = bsProc
				end
			end
		end
	end
end