local _G = getfenv(0)
local ADDON_NAME, addon = ...

if not addon.Retail then return end

local LibStub = _G.LibStub
local CompactRunes = LibStub("AceAddon-3.0"):GetAddon(addon.addonName)
local L = LibStub("AceLocale-3.0"):GetLocale(addon.addonName)

local ipairs = _G.ipairs
local tsort = table.sort

local runes = addon.runes
local runeBars = addon.runeBars

addon.runeOrderings = {
	["Right"] = { 1, 2, 3, 4, 5, 6 },
	["Down"] = { 1, 3, 5, 2, 4, 6 },
	["Left"] = { 2, 4, 6, 1, 3, 5 },
	["Up"] = { 5, 3, 1, 6, 4, 2 },
}
local runeOrderings = addon.runeOrderings

local HEXFMT = "|c%2x%2x%2x%2x"
local function chatColorCode(color)
	local a = 255
	local r = ceil(255 * color.r)
	local g = ceil(255 * color.g)
	local b = ceil(255 * color.b)
	return HEXFMT:format(a, r, g, b)
end

local RuneColor = nil
local RuneCooldownColor = nil
local RuneChatColor = nil

function CompactRunes:UpdateRuneColors()
	RuneColor = self.db.profile.RuneColors["DEFAULT"]
	RuneCooldownColor = self.db.profile.RuneCooldownColors["DEFAULT"]
	RuneChatColor = chatColorCode(self.db.profile.RuneColors["DEFAULT"])
end

function CompactRunes:OrderRunes()
	local newOrder = {}
	for i = 1, #runes do
		newOrder[i] = i
	end
	tsort(newOrder, function(a,b)
		local vala = runes[a].ready and 0 or runes[a].duration - runes[a].timer
		local valb = runes[b].ready and 0 or runes[b].duration - runes[b].timer
		return vala == valb and (a < b) or (vala < valb)
	end)
	return newOrder
end

function CompactRunes:UpdateRuneStatus(runeIndex, barIndex)
	local profile = self.db.profile
	local runeBar = runeBars[barIndex]
	local rune = runes[runeIndex]

	-- if barIndex == 1 then
	-- 	local fmt = "Rune %d -> %d [%s,%s,%s]"
	-- 	self:Print(fmt:format(runeIndex, barIndex, _G.tostring(rune.ready), _G.tostring(rune.duration), _G.tostring(rune.timer)))
	-- end

	local rc, rcc
	runeBar.bar.showing = rune
	runeBar.bar.status = rune.ready and "ready" or "recharging"

	rc = RuneColor
	if profile.runeBackground == 1 then
		rcc = RuneCooldownColor
		rcc.a = profile.runeBackgroundAlpha or rcc.a
	else
		rcc = self.altRuneBkg
	end
	runeBar.bar:SetStatusBarColor(rc.r, rc.g, rc.b, 1)
	runeBar.bar.bg:SetVertexColor(rcc.r, rcc.g, rcc.b, rcc.a)

	runeBar.bar:SetMinMaxValues(0, rune.duration)
	runeBar.bar:SetValue(rune.timer)
	if rune.ready == false then
		runeBar.bar:SetScript("OnUpdate", addon.scripts.onUpdateRune)
		runeBar.bar:SetAlpha(profile.rechargeAlpha)
	else
		runeBar.bar:SetScript("OnUpdate", nil)
		runeBar.bar:SetAlpha(1.0)
		if profile.rechargeTimers ~= "None" then
			runeBar.bar.value:SetText("")
		end
	end
end

function CompactRunes:UpdateRunes()
  -- Update the rune data.
  for i = 1, addon.NUMBER_RUNES do
    self:UpdateRuneCooldown(i)
  end

  -- Update the rune UI.
	local profile = self.db.profile
  if profile.sortRunes then
    local order = self:OrderRunes()
    for i, pos in ipairs(runeOrderings[profile.runeSortOrder]) do
      self:UpdateRuneStatus(order[i], pos)
    end
  else
    for i = 1, addon.NUMBER_RUNES do
      self:UpdateRuneStatus(i, i)
    end
  end
end

function CompactRunes:UpdateRuneCooldown(index, isEnergized)
	local profile = self.db.profile
	local start, duration, ready = _G.GetRuneCooldown(index)
	start = start or 0
	duration = duration or 0
	local rune = runes[index]
	if not rune then return end
	-- TODO: Deal with the energized runes.
	rune.ready = ready or isEnergized == true
	local time = GetTime()

	if profile.runeDebug then
		local remain = start > 0 and (start + duration - time) or -10
		local fmt = "Rune CD [%s,%s,%s,%s,%s, %.1f]"
		self:Print(fmt:format(tostring(index), tostring(start),
			tostring(duration), tostring(ready), tostring(isEnergized), remain or -11))
	end

	rune.start = start
	if rune.ready == false then
		rune.duration = duration
		rune.timer = time - start
	else
		rune.timer = 10
		rune.duration = 10
	end

	-- Updated count of depleted runes
	local depleted = 0
	for i, v in ipairs(runes) do
		if v then
			if not v.ready then
				depleted = depleted + 1
			end
		end
	end
	addon.depletedRunes = depleted
	--local fmt = "Depleted: %s / %s"
	--self:Print(fmt:format(tostring(depleted)))
end
