local _G = getfenv(0)
local ADDON_NAME, addon = ...

local LibStub = _G.LibStub
local CompactRunes = LibStub("AceAddon-3.0"):GetAddon(addon.addonName)
local L = LibStub("AceLocale-3.0"):GetLocale(addon.addonName)

local ipairs = _G.ipairs
local tsort = table.sort

local runes = addon.runes
local runeBars = addon.runeBars
local RuneMapping = addon.RuneMapping

local GetRuneType = _G.GetRuneType

addon.runeOrderings = {
	["BFU"] = { 1, 2, 3, 4, 5, 6 },
	["BUF"] = { 1, 2, 5, 6, 3, 4 },
	["FBU"] = { 3, 4, 1, 2, 5, 6 },
	["UFB"] = { 5, 6, 3, 4, 1, 2 },
}
local runeOrderings = addon.runeOrderings

local HEXFMT = "|c%2x%2x%2x%2x"
local function chatColorCode(color)
	local a = 255
	local r = ceil(255 * color.r)
	local g = ceil(255 * color.g)
	local b = ceil(255 * color.b)
	return HEXFMT:format(a, r, g, b)
end

function CompactRunes:UpdateRuneColors()
end

function CompactRunes:OrderRunes()
	local newOrder = {}
	for i = 1, #runes do
		newOrder[i] = i
	end
	tsort(newOrder, function(a,b)
		if runes[a].baseType == runes[b].baseType then
			local vala = runes[a].ready and 0 or runes[a].duration - runes[a].timer
			local valb = runes[b].ready and 0 or runes[b].duration - runes[b].timer
			return vala == valb and (a < b) or (vala < valb)
		else
			return runes[a].baseType < runes[b].baseType 
		end
	end)
	return newOrder
end

function CompactRunes:UpdateRuneStatus(runeIndex, barIndex)
	local profile = self.db.profile
	local runeBar = runeBars[barIndex]
	local rune = runes[runeIndex]

	-- if barIndex == 1 then
	-- 	local fmt = "Rune %d -> %d [%s,%s,%s]"
	-- 	self:Print(fmt:format(runeIndex, barIndex, _G.tostring(rune.ready), _G.tostring(rune.duration), _G.tostring(rune.timer)))
	-- end

	local rc, rcc
	runeBar.bar.showing = rune
	runeBar.bar.status = rune.ready and "ready" or "recharging"

	local runeType = RuneMapping[rune.type]
	rc = profile.RuneColors[runeType]
	if profile.runeBackground == 1 then
		rcc = profile.RuneCooldownColors[runeType]
		rcc.a = profile.runeBackgroundAlpha or rcc.a
	else
		rcc = self.altRuneBkg
	end
	runeBar.bar:SetStatusBarColor(rc.r, rc.g, rc.b, 1)
	runeBar.bar.bg:SetVertexColor(rcc.r, rcc.g, rcc.b, rcc.a)

	runeBar.bar:SetMinMaxValues(0, rune.duration)
	runeBar.bar:SetValue(rune.timer)
	if rune.ready == false then
		runeBar.bar:SetScript("OnUpdate", addon.scripts.onUpdateRune)
		runeBar.bar:SetAlpha(profile.rechargeAlpha)
	else
		runeBar.bar:SetScript("OnUpdate", nil)
		runeBar.bar:SetAlpha(1.0)
		if profile.rechargeTimers ~= "None" then
			runeBar.bar.value:SetText("")
		end
	end
end

function CompactRunes:UpdateRunes()
  -- Update the rune data.
  for i = 1, addon.NUMBER_RUNES do
	self:UpdateRuneType(i)
    self:UpdateRuneCooldown(i)
  end

  -- Update the rune UI.
	local profile = self.db.profile
  if profile.sortRunes then
    local order = self:OrderRunes()
	local ordering = runeOrderings[profile.runeSortOrder]
    for i, pos in ipairs(ordering) do
      self:UpdateRuneStatus(order[i], pos)
    end
  else
    for i = 1, addon.NUMBER_RUNES do
      self:UpdateRuneStatus(i, i)
    end
  end
end

function CompactRunes:UpdateRuneType(index)
	local rune = runes[index]
	local type = GetRuneType(index)
	runes[index].type = type
end

local function GetShowing(index)
	for i = 1, addon.NUMBER_RUNES do
		local runeBar = runeBars[i]
		if runeBar.bar.showing.index == index then return i end
	end
	return 0
end

function CompactRunes:UpdateRuneCooldown(index, isEnergized)
	local profile = self.db.profile
	local start, duration, ready = _G.GetRuneCooldown(index)
	start = start or 0
	duration = duration or 0
	local rune = runes[index]
	if not rune then return end
	-- TODO: Deal with the energized runes.
	rune.ready = ready or isEnergized == true
	local time = GetTime()

	if profile.runeDebug then
		local remain = start > 0 and (start + duration - time) or -10
		local fmt = "Rune CD [%s [%s], %s,%s,%s,%s,%s, %.1f]"
		self:Print(fmt:format(tostring(index), tostring(GetShowing(index)), tostring(rune.type), tostring(start),
			tostring(duration), tostring(ready), tostring(isEnergized), remain or -11))
	end

	rune.start = start
	if rune.ready == false then
		rune.duration = duration
		rune.timer = time - start
	else
		rune.timer = 10
		rune.duration = 10
	end

	-- Updated count of depleted runes
	local depleted = 0
	for i, v in ipairs(runes) do
		if v then
			if not v.ready then
				depleted = depleted + 1
			end
		end
	end
	addon.depletedRunes = depleted
	--local fmt = "Depleted: %s / %s"
	--self:Print(fmt:format(tostring(depleted)))
end
